//
//  IDIDevice.h
//  iDeviceInterface
//
//  Created by Chris Devor on 7/14/11.
//  Copyright 2011 Macroplant. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MobileDevice.h"
#import "IDDeviceConnectionManager.h"

extern NSString* AFCADDRESS;
extern NSString* AFC2ADDRESS;

@interface IDDeviceAppDirectoryItem : NSObject {
   
   NSString *name;
   NSString* ID;
   NSString* iconPath;
   NSString* appFolder;
   BOOL isFileSharingEnabled;
}
@property(readwrite, retain) NSString *name;
@property(readwrite, retain) NSString *ID;
@property(readwrite, retain) NSString *iconPath;
@property(readwrite, retain) NSString *appFolder;
@property(readwrite)         BOOL      isFileSharingEnabled;

@end


@interface IDIDevice : NSObject {
   
   NSLock* lock_connectionLock;
   
   // When the device is disconnected, this is set so that no future AFC calls are attempted
   BOOL abortNow;
   
   struct am_device *myDevice;
	struct afc_connection *myConnection;
   
   // FileRefs that may be opened in one call, then read/written in another, then closed in another
   // Locked with connectionLock
   // Each object in an NSDictionary:
   //    "path" - NSString, path for file
   //    "fileRef" -- NSNumber(unsigned long long), typecast to afc_file_ref
   //    "mode" - NSNumber(int) - mode passed to AFCFileRefOpen() for read, write, or read-write
   // NOTE: all currently open fileRefs must be part of the same service. If the service changes or is restarted,
   // then the file refs all become invalid.
   NSMutableArray* ma_hangingFileRefs;
   NSString* hangingFilesService;
	
	BOOL isJailbreaked;
   BOOL isConnected;
   
   NSString* deviceName;
   NSMutableArray* appBundles;
   
   NSString* currentService;
   
   NSObject <IDDeviceConnectionManagerDelegate> *delegate;
}
@property(readonly) struct am_device *myDevice;
//@property(readonly) struct afc_connection *myConnection;
@property(readonly) BOOL isJailbreaked;
@property(readonly) BOOL isUsingAFC2;
@property(readonly) BOOL isConnected;
@property (readonly) NSString* currentService;

// Only called by the connection manager on disconnect
@property BOOL abortNow;


@property (readonly) NSString* deviceName;
@property (readonly) NSString* activationState;
@property (readonly) NSString* activationStateAcknowledged;
@property (readonly) NSString* basebandBootloaderVersion;
@property (readonly) NSString* basebandVersion;
@property (readonly) NSString* bluetoothAddress;
@property (readonly) NSString* buildVersion;
@property (readonly) NSString* deviceCertificate;
@property (readonly) NSString* deviceClass;
@property (readonly) NSString* devicePublicKey;
@property (readonly) NSString* firmwareVersion;
@property (readonly) NSString* fostAttached;
@property (readonly) NSString* integratedCircuitCardIdentity;
@property (readonly) NSString* internationalMobileEquipmentIdentity;
@property (readonly) NSString* internationalMobileSubscriberIdentity;
@property (readonly) NSString* modelNumber;
@property (readonly) NSString* phoneNumber;
@property (readonly) NSString* productType;
@property (readonly) NSString* productVersion;
@property (readonly) NSString* protocolVersion;
@property (readonly) NSString* regionInfo;
@property (readonly) NSString* sBLockdownEverRegisteredKey;
@property (readonly) NSString* sIMStatus;
@property (readonly) NSString* serialNumber;
@property (readonly) NSString* somebodySetTimeZone;
@property (readonly) NSString* timeIntervalSince1970;
@property (readonly) NSString* timeZone;
@property (readonly) NSString* timeZoneOffsetFromUTC;
@property (readonly) NSString* trustedHostAttached;
@property (readonly) NSString* uniqueDeviceID;
@property (readonly) NSString* uses24HourClock;
@property (readonly) NSString* wiFiAddress;
@property (readonly) NSString* iTunesHasConnected;

// Is the current service an app connection? otherwise it is AFC or AFC2
- (BOOL) isAppService;

- (id)initWithDevice:(struct am_device *)thisDevice allowAFC2:(BOOL) allowAFC2 delegate:(NSObject <IDDeviceConnectionManagerDelegate>*) del;

- (BOOL) restartSession;
- (BOOL) connectToService:(NSString*) newService;

- (BOOL) copyPathToComputerFromDevice:(NSString *)sourcePathOnDevice destinationOnComputer:(NSString *)destinationPathOnComputer serviceName:(NSString*)service;
- (BOOL) copyPathToDeviceFromComputer:(NSString *)sourcePathOnComputer targetPath:(NSString *) targetPathOnDevice serviceName:(NSString*)service;
- (NSData*) readDataFromFile:(NSString *)sourcePathOnDevice serviceName:(NSString*)service;
- (NSData*) readDataFromFile:(NSString *)sourcePathOnDevice offset:(unsigned long long)offset length:(unsigned long long)length serviceName:(NSString*)service;
- (BOOL) writeData:(NSData*)data toFile:(NSString*)pathOnDevice serviceName:(NSString*)service;
- (BOOL) writeData:(NSData*)data range:(NSRange)range toFile:(NSString*)pathOnDevice serviceName:(NSString*)service;

- (NSArray*) appBundles;
- (NSDictionary *) GetFileInfo:(NSString *)path serviceName:(NSString*)service;

- (BOOL) createNewFolder:(NSString *)folderPath serviceName:(NSString*)service;
- (BOOL) renameFolder:(NSString *)folderPath newFolderName:(NSString *)newFolderName serviceName:(NSString*)service;
- (BOOL) movePath:(NSString*)sourcePath toPath:(NSString*)destPath serviceName:(NSString*)service;
- (BOOL) removePath:(NSString *) folderPath serviceName:(NSString*)service;
- (BOOL) fileExistsOnDevice:(NSString *) thisPath serviceName:(NSString*)service;
- (BOOL) directoryExistsOnDevice:(NSString *)thisPath serviceName:(NSString*)service;

- (BOOL) createLinkAtPath:(NSString*)path targetPath:(NSString*)target serviceName:(NSString*)service;
- (BOOL) createSymbolicLinkAtPath:(NSString*)path targetPath:(NSString*)target serviceName:(NSString*)service;

- (BOOL) isDirectory:(NSString *)path serviceName:(NSString*)service;
- (NSNumber *) getFileSize:(NSString *)path serviceName:(NSString*)service;

- (BOOL) setFileSize:(unsigned long long)size forPath:(NSString*)path serviceName:(NSString*)service;

- (NSArray *) contentsInDirectory:(NSString *)directoryPath includeHiddenFiles:(BOOL)includeHidden serviceName:(NSString*)service;
- (NSArray *) sortedContentsInDirectory:(NSString *)directoryPath includeHiddenFiles:(BOOL)includeHidden serviceName:(NSString*)service;
- (NSArray *) getDirectories:(NSString *)directoryPath includeHidden:(BOOL)includeHidden serviceName:(NSString*)service;
- (NSArray *) getFiles:(NSString *)directoryPath includeHidden:(BOOL)includeHidden serviceName:(NSString*)service;

- (NSDictionary *) getDeviceInfo;
- (NSNumber *) getDeviceCapacity;
- (NSNumber *) getDeviceFreeSpace;

- (NSNumber *) getFileSize:(NSString *)path serviceName:(NSString*)service;
- (NSDate *) getDateModified:(NSString *)path serviceName:(NSString*)service;


// Open a hanging file ref
- (BOOL) openHangingFileForReading:(NSString*)path serviceName:(NSString*)service;
- (BOOL) openHangingFileForWriting:(NSString*)path serviceName:(NSString*)service;
- (BOOL) openHangingFileForReadingAndWriting:(NSString*)path serviceName:(NSString*)service;
- (void) closeHangingFileRef:(NSString*)path serviceName:(NSString*)service;

// Returns empty data on end-of-file, nil if an error occurred
- (NSData*) readHangingFile:(NSString*)path serviceName:(NSString*)service offset:(unsigned long long)offset size:(unsigned long long)size;

// Returns -1 on error, else the actual number of bytes written
- (signed long long) writeHangingFile:(NSString*)path serviceName:(NSString*)service offset:(unsigned long long)offset size:(unsigned long long)size data:(const void*)pData;


@end
